@@ -1,4 +1,5 @@
 # Copyright (C) 2014-2018 Enzien Audio, Ltd.
+# Copyright (C) 2023-2024 Wasted Audio
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,8 +15,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import json
+import random
 import os
 
+from typing import Any, Dict, List, Optional, Set, Tuple
+
 from .HIrConvolution import HIrConvolution
 from .HIrInlet import HIrInlet
 from .HIrLorenz import HIrLorenz
@@ -29,13 +33,15 @@ from .HIrTabwrite import HIrTabwrite
 from .HLangAdc import HLangAdc
 from .HLangBinop import HLangBinop
 from .HLangBiquad import HLangBiquad
+from .HLangDac import HLangDac
 from .HLangDelay import HLangDelay
-# from .HLangIf import HLangIf
+# from .HLangIf import HLangIf  # circular import. moved here
 from .HeavyException import HeavyException
 from .HeavyIrObject import HeavyIrObject
 from .HeavyLangObject import HeavyLangObject
 from .HLangLine import HLangLine
 from .HLangMessage import HLangMessage
+# from .HLangNoise import HLangNoise  # circular import. moved here
 from .HLangPhasor import HLangPhasor
 from .HLangPrint import HLangPrint
 from .HLangReceive import HLangReceive
@@ -56,7 +62,14 @@ from .Connection import Connection
 class HeavyParser:
 
     @classmethod
-    def graph_from_file(clazz, hv_file, graph=None, graph_args=None, path_stack=None, xname=None):
+    def graph_from_file(
+        cls,
+        hv_file: str,
+        graph: Optional[HeavyGraph] = None,
+        graph_args: Optional[Dict] = None,
+        path_stack: Optional[set] = None,
+        xname: Optional[str] = None
+    ) -> HeavyGraph:
         """ Read a graph object from a file.
 
             @param graph  The parent graph of this graph.
@@ -79,10 +92,19 @@ class HeavyParser:
         with open(hv_file, "r") as f:
             json_heavy = json.load(f)
 
-        return HeavyParser.graph_from_object(json_heavy, graph, graph_args, hv_file, path_stack, xname)
+        return cls.graph_from_object(hv_file, json_heavy, path_stack, graph, graph_args, xname)
 
     @classmethod
-    def graph_from_object(clazz, json_heavy, graph=None, graph_args=None, hv_file=None, path_stack=None, xname=None):
+    def graph_from_object(
+        cls,
+        hv_file: str,
+        json_heavy: Dict,
+        path_stack: set,
+        graph: Optional[HeavyGraph] = None,
+        graph_args: Optional[Dict] = None,
+        xname: Optional[str] = None
+    ) -> HeavyGraph:
+
         """ Parse a graph object.
 
             @param graph  The parent graph.
@@ -122,7 +144,7 @@ class HeavyParser:
                 elif o["type"] == "graph":
                     # inline HeavyGraph objects (i.e. subgraphs)
                     # require a different set of initialisation arguments
-                    x = HeavyParser.graph_from_object(o, g, g.args, hv_file, path_stack, xname)
+                    x: Any = cls.graph_from_object(hv_file, o, path_stack, g, g.args, xname)
 
                 else:
                     # resolve the arguments dictionary based on the graph args
@@ -134,7 +156,7 @@ class HeavyParser:
                     # name as the type that we are looking for, don't recurse!
                     abs_path = g.find_path_for_abstraction(o["type"])
                     if abs_path is not None and abs_path not in path_stack:
-                        x = HeavyParser.graph_from_file(
+                        x = cls.graph_from_file(
                             hv_file=abs_path,
                             graph=g,
                             graph_args=args,
@@ -143,8 +165,8 @@ class HeavyParser:
                     # if we know how to handle this object type natively
                     # either as a custom type or as a generic IR object
                     elif HeavyParser.get_class_for_type(o["type"]) is not None:
-                        obj_clazz = HeavyParser.get_class_for_type(o["type"])
-                        x = obj_clazz(o["type"], args, g, o.get("annotations", {}))
+                        obj_cls = HeavyParser.get_class_for_type(o["type"])
+                        x = obj_cls(o["type"], args, g, o.get("annotations", {}))
 
                     # handle generic IR objects
                     elif HeavyIrObject.is_ir(o["type"]):
@@ -172,8 +194,8 @@ class HeavyParser:
             if g.is_root_graph():
                 # add the notification dictionary at the top level
                 e.notes = g.get_notices()
-                e.notes["has_error"] = True
-                e.notes["exception"] = e
+                e.notes.has_error = True
+                e.notes.exception = e
             raise e
 
         if (g.graph is None) or (g.graph.file != g.file):
@@ -184,7 +206,7 @@ class HeavyParser:
         return g
 
     @classmethod
-    def get_class_for_type(clazz, obj_type):
+    def get_class_for_type(cls, obj_type: str) -> Any:
         """ Returns the class which can handle the given object type.
         """
         if HLangUnop.handles_type(obj_type):
@@ -201,13 +223,17 @@ class HLangIf(HeavyLangObject):
     """ Translates HeavyLang object [if] to HeavyIR [if] or [if~].
     """
 
-    def __init__(self, obj_type, args, graph, annotations=None):
-        HeavyLangObject.__init__(self, "if", args, graph,
-                                 num_inlets=2,
-                                 num_outlets=2,
-                                 annotations=annotations)
+    def __init__(
+        self,
+        obj_type: str,
+        args: Dict,
+        graph: HeavyGraph,
+        annotations: Optional[Dict] = None
+    ) -> None:
+        assert obj_type == "if"
+        super().__init__("if", args, graph, num_inlets=2, num_outlets=2, annotations=annotations)
 
-    def reduce(self):
+    def reduce(self) -> Tuple[Set, List]:
         if self.has_inlet_connection_format(["cc", "_c", "c_", "__"]):
             x = HeavyIrObject("__if", self.args)
         elif self.has_inlet_connection_format("ff"):
@@ -223,10 +249,28 @@ class HLangIf(HeavyLangObject):
         return ({x}, self.get_connection_move_list(x))
 
 
-# NOTE(mhroth): these imports are at the end of the file in order to prevent
-# circular import errors
-from .HLangDac import HLangDac
-from .HLangNoise import HLangNoise
+class HLangNoise(HeavyLangObject):
+    """ Handles the HeavyLang "noise" object.
+    """
+
+    def __init__(
+        self,
+        obj_type: str,
+        args: Dict,
+        graph: HeavyGraph,
+        annotations: Optional[Dict] = None
+    ) -> None:
+        assert obj_type == "noise"
+        super().__init__("noise", args, graph, num_inlets=1, num_outlets=1, annotations=annotations)
+
+    def reduce(self) -> Tuple[Set, List]:
+        seed = int(random.uniform(1, 2147483647))  # assign a random 32-bit seed
+        noise_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "./hvlib/noise.hv.json")
+        x = HeavyParser.graph_from_file(noise_path, graph_args={"seed": seed})
+        x.reduce()
+        # TODO(mhroth): deal with control input
+        return ({x}, self.get_connection_move_list(x))
+
 
 # A list of all of the HeavyLang objects and the classes
 # that will translate them into HeavyIR objects.
@@ -254,6 +298,7 @@ LANG_CLASS_DICT = {
     "slice": HLangSlice,
     "__tabread~if": HIrTabread,
     "__tabread~f": HIrTabread,
+    "__tabread_stoppable~f": HIrTabread,
     "__tabreadu~f": HIrTabread,
     "__tabread": HIrTabread,
     "__tabhead~f": HIrTabhead,
