@@ -1,4 +1,5 @@
 # Copyright (C) 2014-2018 Enzien Audio, Ltd.
+# Copyright (C) 2021-2024 Wasted Audio
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,12 +20,17 @@ import time
 import jinja2
 
 from shutil import which
+from typing import Optional
 
 from hvcc.core.hv2ir.HeavyException import HeavyException
 from ..copyright import copyright_manager
 
+from hvcc.interpreters.pd2hv.NotificationEnum import NotificationEnum
+from hvcc.types.compiler import Generator, CompilerResp, CompilerNotif, CompilerMsg, ExternInfo
+from hvcc.types.meta import Meta
 
-class c2js:
+
+class c2js(Generator):
     """Compiles a directory of C source files into javascript. Requires the
     emscripten library to be installed - https://github.com/kripken/emscripten
     """
@@ -53,18 +59,34 @@ class c2js:
         "_hv_table_setLength",
         "_hv_table_getBuffer",
         "_hv_sendMessageToReceiverV",
+        "_hv_sendMessageToReceiverFF",
+        "_hv_sendMessageToReceiverFFF",
         "_malloc"  # Rationale: https://github.com/emscripten-core/emscripten/issues/6882#issuecomment-406745898
     ]
 
     @classmethod
-    def run_emscripten(clazz, c_src_dir, out_dir, patch_name, output_name, post_js_path, should_modularize,
-                       environment, pre_js_path="", binaryen_async=1):
+    def run_emscripten(
+        cls,
+        c_src_dir: str,
+        out_dir: str,
+        patch_name: str,
+        output_name: str,
+        post_js_path: str,
+        should_modularize: int,
+        environment: str,
+        pre_js_path: str = "",
+        binaryen_async: int = 1
+    ) -> str:
         """Run the emcc command to compile C source files to a javascript library.
         """
 
-        emcc_path = which("emcc")
+        # Detect Windows OS, but ignore if running in MingW
+        if os.name == 'nt' and os.environ.get('MSYSTEM') is None:
+            emcc_path = which("emcc.bat")
+        else:
+            emcc_path = which("emcc")
 
-        if not emcc_path:
+        if emcc_path is None:
             raise HeavyException("emcc is not in the PATH")
 
         c_flags = [
@@ -83,26 +105,25 @@ class c2js:
         # compile C files
         for c in c_src_paths:
             obj_path = f"{os.path.splitext(c)[0]}.o"
-            cmd = [emcc_path] + c_flags + ["-c", "-o", obj_path, c]
+            cmd = [emcc_path] + c_flags + ["-c", "-o", obj_path, c]  # type: ignore
             subprocess.check_output(cmd)  # run emscripten
             obj_paths += [obj_path]
 
         # compile C++ files
         for cpp in cpp_src_paths:
             obj_path = f"{os.path.splitext(cpp)[0]}.o"
-            cmd = [emcc_path] + c_flags + ["-std=c++11"] + ["-c", "-o", obj_path, cpp]
+            cmd = [emcc_path] + c_flags + ["-std=c++11"] + ["-c", "-o", obj_path, cpp]  # type: ignore
             subprocess.check_output(cmd)  # run emscripten
             obj_paths += [obj_path]
 
         # exported heavy api methods
-        hv_api_defs = ", ".join([f"\"{x.format(patch_name)}\"" for x in c2js.__HV_API])
+        hv_api_defs = ", ".join([f"\"{x.format(patch_name)}\"" for x in cls.__HV_API])
 
         # output path
         wasm_js_path = os.path.join(out_dir, f"{output_name}.js")
 
         linker_flags = [
             "-O3",
-            "--memory-init-file", "0",
             "-s", "RESERVED_FUNCTION_POINTERS=2",
             "-s", "DEFAULT_LIBRARY_FUNCS_TO_INCLUDE=$addFunction",
             "-s", f"EXPORTED_FUNCTIONS=[{hv_api_defs.format(patch_name)}]",
@@ -122,9 +143,9 @@ class c2js:
             ]
 
         # include C/C++ obj files in js library
-        cmd = [emcc_path] + obj_paths + linker_flags
+        cmd = [emcc_path] + obj_paths + linker_flags  # type: ignore
         subprocess.check_output(  # WASM
-            cmd + [
+            cmd + [  # type: ignore
                 "-s", "WASM=1",
                 "-s", f"EXPORT_NAME='{output_name}_Module'",
                 "-o", wasm_js_path
@@ -137,13 +158,28 @@ class c2js:
         return wasm_js_path
 
     @classmethod
-    def compile(clazz, c_src_dir, out_dir, externs, patch_name=None, patch_meta: dict = None,
-                num_input_channels=0, num_output_channels=0, copyright=None, verbose=False):
+    def compile(
+        cls,
+        c_src_dir: str,
+        out_dir: str,
+        externs: ExternInfo,
+        patch_name: Optional[str] = None,
+        patch_meta: Meta = Meta(),
+        num_input_channels: int = 0,
+        num_output_channels: int = 0,
+        copyright: Optional[str] = None,
+        verbose: Optional[bool] = False
+    ) -> CompilerResp:
 
         tick = time.time()
 
-        parameter_list = externs["parameters"]["in"]
-        event_list = externs["events"]["in"]
+        parameter_list = externs.parameters.inParam
+        parameter_out_list = externs.parameters.outParam
+        event_list = externs.events.inEvent
+        event_out_list = externs.events.outEvent
+
+        midi_list = externs.midi.inMidi
+        midi_out_list = externs.midi.outMidi
 
         out_dir = os.path.join(out_dir, "js")
         patch_name = patch_name or "heavy"
@@ -171,15 +207,15 @@ class c2js:
                     name=patch_name,
                     copyright=copyright_js,
                     externs=externs,
-                    pool_sizes_kb=externs["memoryPoolSizesKb"]))
+                    pool_sizes_kb=externs.memoryPoolSizesKb))
 
-            js_path = c2js.run_emscripten(c_src_dir=c_src_dir,
-                                          out_dir=out_dir,
-                                          patch_name=patch_name,
-                                          output_name=patch_name,
-                                          post_js_path=post_js_path,
-                                          should_modularize=1,
-                                          environment="web")
+            js_path = cls.run_emscripten(c_src_dir=c_src_dir,
+                                         out_dir=out_dir,
+                                         patch_name=patch_name,
+                                         output_name=patch_name,
+                                         post_js_path=post_js_path,
+                                         should_modularize=1,
+                                         environment="web")
 
             # delete temporary files
             os.remove(post_js_path)
@@ -192,7 +228,11 @@ class c2js:
                     name=patch_name,
                     includes=[f"./{js_out_file}"],
                     parameters=parameter_list,
+                    parameters_out=parameter_out_list,
                     events=event_list,
+                    events_out=event_out_list,
+                    midi=midi_list,
+                    midi_out=midi_out_list,
                     copyright=copyright_html))
 
             # generate heavy js worklet from template
@@ -204,7 +244,7 @@ class c2js:
                     name=patch_name,
                     copyright=copyright_js,
                     externs=externs,
-                    pool_sizes_kb=externs["memoryPoolSizesKb"]))
+                    pool_sizes_kb=externs.memoryPoolSizesKb))
 
             pre_js_path = os.path.join(out_dir, "hv_worklet_start.js")
             with open(pre_js_path, "w") as f:
@@ -212,52 +252,43 @@ class c2js:
                     name=patch_name,
                     copyright=copyright_js,
                     externs=externs,
-                    pool_sizes_kb=externs["memoryPoolSizesKb"]))
+                    pool_sizes_kb=externs.memoryPoolSizesKb))
 
-            js_path = c2js.run_emscripten(c_src_dir=c_src_dir,
-                                          out_dir=out_dir,
-                                          patch_name=patch_name,
-                                          output_name=f"{patch_name}_AudioLibWorklet",
-                                          post_js_path=post_js_path,
-                                          should_modularize=0,
-                                          environment="worker",
-                                          pre_js_path=pre_js_path,
-                                          binaryen_async=0)
+            js_path = cls.run_emscripten(c_src_dir=c_src_dir,
+                                         out_dir=out_dir,
+                                         patch_name=patch_name,
+                                         output_name=f"{patch_name}_AudioLibWorklet",
+                                         post_js_path=post_js_path,
+                                         should_modularize=0,
+                                         environment="shell,worker",
+                                         pre_js_path=pre_js_path,
+                                         binaryen_async=0)
 
             # delete temporary files
             os.remove(post_js_path)
             os.remove(pre_js_path)
 
-            return {
-                "stage": "c2js",
-                "notifs": {
-                    "has_error": False,
-                    "exception": None,
-                    "warnings": [],
-                    "errors": []
-                },
-                "in_dir": c_src_dir,
-                "in_file": "",
-                "out_dir": out_dir,
-                "out_file": js_out_file,
-                "compile_time": time.time() - tick
-            }
+            return CompilerResp(
+                stage="c2js",
+                in_dir=c_src_dir,
+                out_dir=out_dir,
+                out_file=js_out_file,
+                compile_time=time.time() - tick
+            )
 
         except Exception as e:
-            return {
-                "stage": "c2js",
-                "notifs": {
-                    "has_error": True,
-                    "exception": e,
-                    "warnings": [],
-                    "errors": [{
-                        "enum": -1,
-                        "message": str(e)
-                    }]
-                },
-                "in_dir": c_src_dir,
-                "in_file": "",
-                "out_dir": out_dir,
-                "out_file": "",
-                "compile_time": time.time() - tick
-            }
+            return CompilerResp(
+                stage="c2js",
+                notifs=CompilerNotif(
+                    has_error=True,
+                    exception=e,
+                    warnings=[],
+                    errors=[CompilerMsg(
+                        enum=NotificationEnum.ERROR_EXCEPTION,
+                        message=str(e)
+                    )]
+                ),
+                in_dir=c_src_dir,
+                out_dir=out_dir,
+                compile_time=time.time() - tick
+            )
