@@ -1,4 +1,5 @@
 # Copyright (C) 2014-2018 Enzien Audio, Ltd.
+# Copyright (C) 2023-2024 Wasted Audio
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,8 +14,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Callable, Dict, List
+
 from .HeavyObject import HeavyObject
 
+from hvcc.types.IR import IRObjectdict, IROnMessage
+
 
 class ControlPrint(HeavyObject):
     """Prints the first value in a message to the console"""
@@ -22,29 +27,36 @@ class ControlPrint(HeavyObject):
     preamble = "cPrint"
 
     @classmethod
-    def get_C_header_set(clazz):
+    def get_C_header_set(cls) -> set:
         return {"HvControlPrint.h"}
 
     @classmethod
-    def get_C_file_set(clazz):
+    def get_C_file_set(cls) -> set:
         return {"HvControlPrint.h", "HvControlPrint.c"}
 
     @classmethod
-    def get_C_init(clazz, obj_type, obj_id, args):
+    def get_C_init(cls, obj_type: str, obj_id: str, args: Dict) -> List[str]:
         return []
 
     @classmethod
-    def get_C_free(clazz, obj_type, obj_id, args):
+    def get_C_free(cls, obj_type: str, obj_id: str, args: Dict) -> List[str]:
         return []
 
     @classmethod
-    def get_C_onMessage(clazz, obj_type, obj_id, inlet_index, args):
+    def get_C_onMessage(cls, obj_type: str, obj_id: str, inlet_index: int, args: Dict) -> List[str]:
         return [f"cPrint_onMessage(_c, m, \"{args['label']}\");"]
 
     @classmethod
-    def get_C_decl(clazz, obj_type, obj_id, args):
+    def get_C_decl(cls, obj_type: str, obj_id: str, args: Dict) -> List[str]:
         return []
 
     @classmethod
-    def get_C_impl(clazz, obj_type, obj_id, on_message_list, obj_class_dict):
+    def get_C_impl(
+        cls,
+        obj_type: str,
+        obj_id: str,
+        on_message_list: List[List[IROnMessage]],
+        get_obj_class: Callable,
+        objects: Dict[str, IRObjectdict]
+    ) -> List[str]:
         return []
