@@ -1,4 +1,5 @@
 # Copyright (C) 2014-2018 Enzien Audio, Ltd.
+# Copyright (C) 2023-2024 Wasted Audio
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,8 +14,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Dict, List
+
 from .HeavyObject import HeavyObject
 
+from hvcc.types.IR import IRSignalList
+
 
 class SignalBiquad(HeavyObject):
     """Handles the biquad~ object.
@@ -24,15 +29,15 @@ class SignalBiquad(HeavyObject):
     preamble = "sBiquad"
 
     @classmethod
-    def get_C_header_set(clazz):
+    def get_C_header_set(cls) -> set:
         return {"HvSignalBiquad.h"}
 
     @classmethod
-    def get_C_file_set(clazz):
+    def get_C_file_set(cls) -> set:
         return {"HvSignalBiquad.h", "HvSignalBiquad.c"}
 
     @classmethod
-    def get_C_def(clazz, obj_type, obj_id):
+    def get_C_def(cls, obj_type: str, obj_id: str) -> List[str]:
         if obj_type == "__biquad_k~f":
             return [f"SignalBiquad_k sBiquad_k_{obj_id};"]
         elif obj_type == "__biquad~f":
@@ -41,7 +46,7 @@ class SignalBiquad(HeavyObject):
             raise Exception()
 
     @classmethod
-    def get_C_init(clazz, obj_type, obj_id, args):
+    def get_C_init(cls, obj_type: str, obj_id: str, args: Dict) -> List[str]:
         if obj_type == "__biquad_k~f":
             return ["sBiquad_k_init(&sBiquad_k_{0}, {1}f, {2}f, {3}f, {4}f, {5}f);".format(
                 obj_id,
@@ -56,28 +61,28 @@ class SignalBiquad(HeavyObject):
             raise Exception()
 
     @classmethod
-    def get_C_free(clazz, obj_type, obj_id, args):
+    def get_C_free(cls, obj_type: str, obj_id: str, args: Dict) -> List[str]:
         return []
 
     @classmethod
-    def get_C_onMessage(clazz, obj_type, obj_id, inlet_index, args):
-        return ["sBiquad_k_onMessage(&Context(_c)->sBiquad_k_{0}, {1}, m);".format(
-            obj_id,
-            inlet_index)]
+    def get_C_onMessage(cls, obj_type: str, obj_id: str, inlet_index: int, args: Dict) -> List[str]:
+        return [f"sBiquad_k_onMessage(&Context(_c)->sBiquad_k_{obj_id}, {inlet_index}, m);"]
 
     @classmethod
-    def get_C_process(clazz, process_dict, obj_type, obj_id, args):
+    def get_C_process(cls, process_dict: IRSignalList, obj_type: str, obj_id: str, args: Dict) -> List[str]:
         if obj_type == "__biquad_k~f":
             return [
                 "__hv_biquad_k_f(&sBiquad_k_{0}, VIf({1}), VOf({2}));".format(
-                    process_dict["id"],
-                    HeavyObject._c_buffer(process_dict["inputBuffers"][0]),
-                    HeavyObject._c_buffer(process_dict["outputBuffers"][0])
+                    process_dict.id,
+                    cls._c_buffer(process_dict.inputBuffers[0]),
+                    cls._c_buffer(process_dict.outputBuffers[0])
                 )]
         elif obj_type == "__biquad~f":
             return [
                 "__hv_biquad_f(&sBiquad_s_{0}, {1}, {2});".format(
-                    process_dict["id"],
-                    ", ".join([f"VIf({HeavyObject._c_buffer(b)})" for b in process_dict["inputBuffers"]]),
-                    ", ".join([f"VOf({HeavyObject._c_buffer(b)})" for b in process_dict["outputBuffers"]])
+                    process_dict.id,
+                    ", ".join([f"VIf({cls._c_buffer(b)})" for b in process_dict.inputBuffers]),
+                    ", ".join([f"VOf({cls._c_buffer(b)})" for b in process_dict.outputBuffers])
                 )]
+        else:
+            raise Exception(f"Incorrect obj_type {obj_type} for SignalBiquad")
