@@ -1,4 +1,5 @@
 # Copyright (C) 2014-2018 Enzien Audio, Ltd.
+# Copyright (C) 2023-2024 Wasted Audio
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,8 +14,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Dict, List
+
 from .HeavyObject import HeavyObject
 
+from hvcc.types.IR import IRSignalList
+
 
 class SignalSample(HeavyObject):
 
@@ -22,38 +27,36 @@ class SignalSample(HeavyObject):
     preamble = "sSample"
 
     @classmethod
-    def get_C_header_set(clazz):
+    def get_C_header_set(cls) -> set:
         return {"HvSignalSample.h"}
 
     @classmethod
-    def get_C_file_set(clazz):
+    def get_C_file_set(cls) -> set:
         return {"HvSignalSample.h", "HvSignalSample.c"}
 
     @classmethod
-    def get_C_init(clazz, obj_type, obj_id, args):
-        return [
-            "sSample_init(&sSample_{0});".format(obj_id)
-        ]
+    def get_C_init(cls, obj_type: str, obj_id: str, args: Dict) -> List[str]:
+        return [f"sSample_init(&sSample_{obj_id});"]
 
     @classmethod
-    def get_C_free(clazz, obj_type, obj_id, args):
+    def get_C_free(cls, obj_type: str, obj_id: str, args: Dict) -> List[str]:
         return []  # nothing to free
 
     @classmethod
-    def get_C_onMessage(clazz, obj_type, obj_id, inlet_index, args):
+    def get_C_onMessage(cls, obj_type: str, obj_id: str, inlet_index: int, args: Dict) -> List[str]:
         return [
             "{0}_onMessage(_c, &Context(_c)->{0}_{1}, {2}, m);".format(
-                clazz.preamble,
+                cls.preamble,
                 obj_id,
                 inlet_index)
         ]
 
     @classmethod
-    def get_C_process(clazz, process_dict, obj_type, obj_id, args):
+    def get_C_process(cls, process_dict: IRSignalList, obj_type: str, obj_id: str, args: Dict) -> List[str]:
         return [
             "__hv_sample_f(this, &sSample_{0}, VIf({1}), &{2}_{0}_sendMessage);".format(
-                process_dict["id"],
-                HeavyObject._c_buffer(process_dict["inputBuffers"][0]),
-                clazz.preamble
+                process_dict.id,
+                cls._c_buffer(process_dict.inputBuffers[0]),
+                cls.preamble
             )
         ]
