@@ -1,4 +1,5 @@
 # Copyright (C) 2014-2018 Enzien Audio, Ltd.
+# Copyright (C) 2023 Wasted Audio
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,20 +14,28 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Optional, List, Dict
+
 from .PdObject import PdObject
 
 
 class PdUnpackObject(PdObject):
-    def __init__(self, obj_type, obj_args=None, pos_x=0, pos_y=0):
+    def __init__(
+        self,
+        obj_type: str,
+        obj_args: Optional[List] = None,
+        pos_x: int = 0,
+        pos_y: int = 0
+    ) -> None:
         assert obj_type == "unpack"
-        PdObject.__init__(self, obj_type, obj_args, pos_x, pos_y)
+        super().__init__(obj_type, obj_args, pos_x, pos_y)
 
         if len(self.obj_args) == 0:
             self.add_error("Unpack requires at least one argument.")
         if not (set(self.obj_args) <= set(["f", "s"])):
             self.add_warning("Heavy only supports arguments 'f' and 's' to unpack.")
 
-    def to_hv(self):
+    def to_hv(self) -> Dict:
         """ Creates a graph dynamically based on the number of arguments.
 
             [inlet                                                ]
@@ -36,7 +45,7 @@ class PdUnpackObject(PdObject):
             [outlet_0]               ...    [outlet_N-1]
         """
 
-        hv_graph = {
+        hv_graph: Dict = {
             "type": "graph",
             "imports": [],
             "args": [],
