@@ -17,24 +17,33 @@ import sys
 import argparse
 import json
 
-import hvcc.core.hv2ir.HeavyLangObject as HeavyLangObject
-import hvcc.interpreters.pd2hv.PdParser as PdParser
+from hvcc.core.hv2ir.HeavyLangObject import HeavyLangObject
+from hvcc.interpreters.pd2hv.PdParser import PdParser
+from hvcc.types.meta import Meta, DPF, Daisy
 
 
-def main():
+gens = {
+    "dpf": DPF,
+    "daisy": Daisy
+}
+
+
+def main() -> None:
     parser = argparse.ArgumentParser(
         description="Some separate heavy utilities, wrapped into a single app")
     subparsers = parser.add_subparsers(dest="command")
     subparsers.add_parser("pdobjects", help="list supported Pure Data objects")
     subparser_hvhash = subparsers.add_parser("hvhash", help="print the heavy hash of the input string")
     subparser_hvhash.add_argument("string")
+    subparser_meta = subparsers.add_parser("metaschema", help="show JSON Schema of the Meta() model")
+    subparser_meta.add_argument("-g", "--generator", help="Choose supported generator: DPF, Daisy")
 
     parsed_args = parser.parse_args(args=None if sys.argv[1:] else ['--help'])
     args = vars(parsed_args)
 
     command = args.pop("command")
     if command == "pdobjects":
-        obj_list = PdParser.PdParser.get_supported_objects()
+        obj_list = PdParser.get_supported_objects()
         obj_list.sort()
 
         obj_dict = {
@@ -43,7 +52,14 @@ def main():
         }
         print(json.dumps(obj_dict, indent=4))
     elif command == "hvhash":
-        print("0x{0:X}".format(HeavyLangObject.HeavyLangObject.get_hash(args.get('string'))))
+        print("0x{0:X}".format(HeavyLangObject.get_hash(args['string'])))
+    elif command == "metaschema":
+        generator = args.get('generator')
+        if generator is not None:
+            model = gens[generator.lower()]()
+        else:
+            model = Meta
+        print(json.dumps(model.model_json_schema(), indent=4))
     else:
         pass
 
